@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -116,6 +116,7 @@ static void
 sg_fs_stats_item_init(sg_fs_stats *d) {
 
 	d->device_name = NULL;
+	d->device_canonical = NULL;
 	d->fs_type = NULL;
 	d->mnt_point = NULL;
 }
@@ -124,6 +125,7 @@ static sg_error
 sg_fs_stats_item_copy(const sg_fs_stats *s, sg_fs_stats *d) {
 
 	if( SG_ERROR_NONE != sg_update_string(&d->device_name, s->device_name) ||
+	    SG_ERROR_NONE != sg_update_string(&d->device_canonical, s->device_canonical) ||
 	    SG_ERROR_NONE != sg_update_string(&d->fs_type, s->fs_type) ||
 	    SG_ERROR_NONE != sg_update_string(&d->mnt_point, s->mnt_point) ) {
 		RETURN_FROM_PREVIOUS_ERROR( "disk", sg_get_error() );
@@ -178,6 +180,10 @@ sg_fs_stats_item_compare(const sg_fs_stats *a, const sg_fs_stats *b) {
 	if( 0 != rc )
 		return rc;
 
+	rc = strcmp(a->device_canonical, b->device_canonical);
+	if( 0 != rc )
+		return rc;
+
 	rc = strcmp(a->mnt_point, b->mnt_point);
 	if( 0 != rc )
 		return rc;
@@ -192,6 +198,7 @@ sg_fs_stats_item_compare(const sg_fs_stats *a, const sg_fs_stats *b) {
 static void
 sg_fs_stats_item_destroy(sg_fs_stats *d) {
 	free(d->device_name);
+	free(d->device_canonical);
 	free(d->fs_type);
 	free(d->mnt_point);
 }
@@ -594,7 +601,9 @@ sg_set_valid_filesystems(const char *valid_fs[]) {
 	/* XXX this must be locked - from caller ? */
 	if( valid_fs && *valid_fs ) {
 
-		while( valid_fs[num_new_valid_fs++] ) {} /* Note: post-inc is intensional */
+		while( valid_fs[num_new_valid_fs] ) {
+			num_new_valid_fs++;
+		}
 
 		new_valid_fs = calloc( num_new_valid_fs + 1, sizeof(char *) );
 		if( 0 == new_valid_fs ) {
@@ -667,10 +676,10 @@ getdrvent(LPTSTR vol, unsigned check_mask, struct sg_win32_drvent *buf) {
 
 	if( check_mask & ( 1 << buf->drive_type ) ) {
 		return GetVolumeInformation( vol,
-					     &buf->volume_name_buf, sizeof(buf->volume_name_buf),
+					     buf->volume_name_buf, sizeof(buf->volume_name_buf),
 					     &buf->volume_serial, &buf->max_component_len,
 					     &buf->fs_flags,
-					     &buf->filesys_name_buf, sizeof(buf->filesys_name_buf) );
+					     buf->filesys_name_buf, sizeof(buf->filesys_name_buf) );
 	}
 
 	return 1;
@@ -807,7 +816,7 @@ sg_get_fs_list_int(sg_vector **fs_stats_vector_ptr) {
 #  endif
 # endif
 # ifndef MNT_MNTTAB
-	static const char *mnttabs[] = { "/etc/mnttab", "/etc/mtab" };
+	static const char *mnttabs[] = { "/proc/mounts", "/etc/mnttab", "/etc/mtab" };
 	unsigned i;
 # endif
 #elif defined(WIN32)
@@ -971,12 +980,14 @@ sg_get_fs_list_int(sg_vector **fs_stats_vector_ptr) {
 		RETURN_WITH_SET_ERROR("disk", SG_ERROR_GETMNTINFO, "GetLogicalDriveStrings");
 	}
 
-	if (!(GetLogicalDriveStrings(BUFSIZE-1, lp_buf))) {
+	if (!(GetLogicalDriveStrings(BUFSIZE-1, buf))) {
 		RETURN_WITH_SET_ERROR("disk", SG_ERROR_GETMNTINFO, "GetLogicalDriveStrings");
 	}
 
 	SetLastError(0);
 
+	int rc;
+
 #define NEED_VECTOR_UPDATE
 
 #undef VECTOR_UPDATE_ERROR_CLEANUP
@@ -986,9 +997,9 @@ sg_get_fs_list_int(sg_vector **fs_stats_vector_ptr) {
 #define SG_FS_LOOP_COND '\0' != *p
 #define SG_FS_LOOP_ITER p += strlen(p) + 1, rc = getdrvent(p, (1 << DRIVE_FIXED)|(1<<DRIVE_RAMDISK), &mp )
 
-#define SG_MP_FSTYPENAME	mp.filesys_name_buf
-#define SG_MP_DEVNAME		p
-#define SG_MP_MOUNTP		p
+#define SG_FS_FSTYPENAME	mp.filesys_name_buf
+#define SG_FS_DEVNAME		p
+#define SG_FS_MOUNTP		p
 #define SG_FS_DEVTYPE		sg_fs_unknown
 
 #else
@@ -1005,6 +1016,7 @@ sg_get_fs_list_int(sg_vector **fs_stats_vector_ptr) {
 	for( SG_FS_LOOP_INIT; SG_FS_LOOP_COND; SG_FS_LOOP_ITER ) {
 
 		sg_error upderr;
+		struct stat statbuf;
 
 #ifdef WIN32
 		if( !mp.volume_serial ) /* skip unformatted disks - XXX recheck this for remote drives ... */
@@ -1015,12 +1027,31 @@ sg_get_fs_list_int(sg_vector **fs_stats_vector_ptr) {
 		VECTOR_UPDATE(fs_stats_vector_ptr, num_fs + 1, fs_ptr, sg_fs_stats);
 #endif
 
+		TRACE_LOG_FMT("disk", "adding mount point \"%s\" for device \"%s\"", SG_FS_MOUNTP, SG_FS_DEVNAME);
 		if( ( ( upderr = sg_update_string( &fs_ptr[num_fs].device_name, SG_FS_DEVNAME ) ) != SG_ERROR_NONE ) ||
+		    ( ( upderr = sg_update_string( &fs_ptr[num_fs].device_canonical, SG_FS_DEVNAME ) ) != SG_ERROR_NONE ) ||
 		    ( ( upderr = sg_update_string( &fs_ptr[num_fs].fs_type, SG_FS_FSTYPENAME ) ) != SG_ERROR_NONE ) ||
 		    ( ( upderr = sg_update_string( &fs_ptr[num_fs].mnt_point, SG_FS_MOUNTP ) ) != SG_ERROR_NONE ) ) {
 			RETURN_FROM_PREVIOUS_ERROR( "disk", upderr );
 		}
 
+		while( ( -1 != lstat(fs_ptr[num_fs].device_canonical, &statbuf) ) && S_ISLNK(statbuf.st_mode) ) {
+			char lnktgt[PATH_MAX+1], *p;
+			TRACE_LOG_FMT("disk", "\"%s\" is symlink - resolving ...", fs_ptr[num_fs].device_canonical);
+			p = realpath(fs_ptr[num_fs].device_canonical, lnktgt);
+			if( NULL == p ) {
+				/* XXX throw error?
+				 * fs_ptr[num_fs].device_canonical might be in undefined state (depends on dirnam behavior) */
+				break;
+			}
+			if( ( upderr = sg_lupdate_string( &fs_ptr[num_fs].device_canonical, lnktgt, sizeof(lnktgt) ) ) != SG_ERROR_NONE ) {
+				RETURN_FROM_PREVIOUS_ERROR( "disk", upderr );
+			}
+		}
+		TRACE_LOG_FMT("disk", "\"%s\" is not a symlink", fs_ptr[num_fs].device_canonical);
+		/* avoid fs. bail out when last entry is no link */
+		errno = 0;
+
 		fs_ptr[num_fs].device_type = SG_FS_DEVTYPE;
 		fs_ptr[num_fs].systime = now;
 
@@ -1168,7 +1199,7 @@ sg_fill_fs_stat_int(sg_fs_stats *fs_stats_ptr) {
 	fs_stats_ptr->total_blocks = fs_stats_ptr->block_size * total_clusters;
 	fs_stats_ptr->free_blocks  = (long long)free_clusters;
 	fs_stats_ptr->avail_blocks =
-	fs_stats_ptr->used_blocks  = fs_stats_ptr->total_block - fs_stats_ptr->free_blocks;
+	fs_stats_ptr->used_blocks  = fs_stats_ptr->total_blocks - fs_stats_ptr->free_blocks;
 #else
 	RETURN_WITH_SET_ERROR("disk", SG_ERROR_UNSUPPORTED, OS_TYPE);
 #define UNSUPPORTED
