@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -346,7 +346,7 @@ sg_get_mem_stats_int(sg_mem_stats *mem_stats_buf) {
 
 	mem_stats_buf->free = memstats.ullAvailPhys;
 	mem_stats_buf->total = memstats.ullTotalPhys;
-	mem_stats_buf->used = mem_stat.total - mem_stat.free;
+	mem_stats_buf->used = mem_stats_buf->total - mem_stats_buf->free;
 	if(read_counter_large(SG_WIN32_MEM_CACHE, &mem_stats_buf->cache))
 		mem_stats_buf->cache = 0;
 #elif defined(HAVE_STRUCT_VMTOTAL)
