@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -27,6 +27,10 @@
 #endif
 #include "tools.h"
 
+#ifdef _WIN32
+#include <iphlpapi.h>
+#endif
+
 static void
 sg_network_io_stats_item_init(sg_network_io_stats *d) {
 	memset( d, 0, sizeof(*d) );
@@ -784,7 +788,13 @@ sg_get_network_iface_stats_int(sg_vector **network_iface_vector_ptr){
 		network_iface_stat[ifaces].systime = now;
 
 		memset(&ifmed, 0, sizeof(struct ifmediareq));
-		sg_strlcpy(ifmed.ifm_name, net_ptr->ifa_name, sizeof(ifmed.ifm_name));
+#ifdef HAVE_STRLCPY
+		strlcpy(ifmed.ifm_name, net_ptr->ifa_name, sizeof(ifmed.ifm_name));
+#elif defined(HAVE_STRNCPY)
+		strncpy(ifmed.ifm_name, net_ptr->ifa_name, sizeof(ifmed.ifm_name)-1);
+#else
+		(void)snprintf(ifmed.ifm_name, sizeof(ifmed.ifm_name)-1, "%s", net_ptr->ifa_name);
+#endif
 		if(ioctl(sock, SIOCGIFMEDIA, (caddr_t)&ifmed) == -1){
 			/* Not all interfaces support the media ioctls. */
 			goto skip;
@@ -1367,11 +1377,11 @@ skip:
 	for (i=0; i < ifaces; ++i) {
 		int no = 2, j;
 		for(j=i+1; j<ifaces; ++j) {
-			if(strcmp(network_io_ptr[i].interface_name, network_io_ptr[j].interface_name) == 0) {
+			if(strcmp(network_iface_stat[i].interface_name, network_iface_stat[j].interface_name) == 0) {
 				if(snprintf(buf, sizeof(buf), " #%d", no) < 0) {
 					break;
 				}
-				if(sg_concat_string(&network_io_ptr[i].interface_name, buf) != 0) {
+				if(sg_concat_string(&network_iface_stat[i].interface_name, buf) != 0) {
 					RETURN_FROM_PREVIOUS_ERROR( "network", sg_get_error() );
 				}
 
