@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -186,15 +186,11 @@ typedef bool _Bool;
 #  include <process.h>
 # endif
 # include <windows.h>
+# include <iphlpapi.h>
+# include <lm.h>
 # ifdef HAVE_PDH_H
 #  include <pdh.h>
 # endif
-# ifdef HAVE_IPHLPAPI_H
-#  include <Iphlpapi.h>
-# endif
-# ifdef HAVE_LM_H
-#  include <lm.h>
-# endif
 # include "win32.h"
 #else
 # ifdef HAVE_SYS_SOCKET_H
@@ -503,18 +499,6 @@ __sg_private int endmntent(FILE *f);
 #  endif
 #endif
 
-#ifndef HAVE_STRLCAT
-__sg_export size_t sg_strlcat(char *dst, const char *src, size_t siz);
-#else
-#define sg_strlcat strlcat
-#endif
-
-#ifndef HAVE_STRLCPY
-__sg_export size_t sg_strlcpy(char *dst, const char *src, size_t siz);
-#else
-#define sg_strlcpy strlcpy
-#endif
-
 #if defined(HAVE_STRNLEN) || (defined(HAVE_XOPEN_SOURCE) && (HAVE_XOPEN_SOURCE>=700))
 #define sg_strnlen strnlen
 #else
@@ -540,10 +524,27 @@ __sg_private char *sg_f_read_line(FILE *f, char *linebuf, size_t buf_size, const
 #if defined(__NEED_SG_GET_SYS_PAGE_SIZE)
 static ssize_t sys_page_size;
 
+#ifdef _WIN32
+# include <windows.h>
+static inline long getpagesize (void) {
+	static long g_pagesize = 0;
+	if (! g_pagesize) {
+		SYSTEM_INFO system_info;
+		GetSystemInfo (&system_info);
+		g_pagesize = system_info.dwPageSize;
+	}
+	return g_pagesize;
+}
+#endif
+
 static inline ssize_t
 sg_get_sys_page_size(void) {
 	if( 0 == sys_page_size ) {
+#ifdef _WIN32
+		if( ( sys_page_size = getpagesize() ) == -1 ) {
+#else
 		if( ( sys_page_size = sysconf(_SC_PAGESIZE) ) == -1 ) {
+#endif
 			SET_ERROR_WITH_ERRNO("tools", SG_ERROR_SYSCONF, "_SC_PAGESIZE");
 		}
 	}
