@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
- * 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -121,7 +121,7 @@ sg_get_user_stats_int(sg_vector **user_stats_vector_ptr) {
 			for( i = 0; i < entries_read; ++i ) {
 				/* assert(tmp_buf != NULL); */
 				if( tmp_buf == NULL ) {
-					sg_set_error(SG_ERROR_PERMISSION, "User list");
+					SET_ERROR("user", SG_ERROR_PERMISSION, "User list");
 					ERROR_LOG("user", "Permission denied fetching user details");
 					break; /* XXX break and not return? */
 				}
