@@ -16,7 +16,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -544,8 +545,11 @@ display_data(int colors) {
 		disk_stat_ptr = stats.fs_stats;
 		for(counter=0;counter<stats.fs_entries;counter++){
 			char name[20];
+			float used_pct;
+
 			strncpy(name, disk_stat_ptr->mnt_point, sizeof(name));
 			name[sizeof(name)-1] = '\0'; /* strncpy doesn't terminate longer strings */
+
 			move(line, 42);
 			printw("%s", name);
 			move(line, 62);
@@ -554,13 +558,19 @@ display_data(int colors) {
 			}
 			printw("%7s", size_conv(disk_stat_ptr->avail));
 			move(line, 73);
-			if(colors && 100.00 * ((float) disk_stat_ptr->used / (float) (disk_stat_ptr->used + disk_stat_ptr->avail)) > THRESHOLD_ALERT_DISK) {
-				attron(A_STANDOUT);
-				attron(A_BOLD);
-			} else if (colors && 100.00 * ((float) disk_stat_ptr->used / (float) (disk_stat_ptr->used + disk_stat_ptr->avail)) > THRESHOLD_WARN_DISK) {
-				attron(A_BOLD);
+			used_pct = ((float) disk_stat_ptr->used / (float) (disk_stat_ptr->used + disk_stat_ptr->avail));
+			if (isnan(used_pct)) {
+				printw("%6s", "-");
+			} else {
+				used_pct *= 100;
+				if(colors && used_pct > THRESHOLD_ALERT_DISK) {
+					attron(A_STANDOUT);
+					attron(A_BOLD);
+				} else if (colors && used_pct > THRESHOLD_WARN_DISK) {
+					attron(A_BOLD);
+				}
+				printw("%6.2f%%", used_pct);
 			}
-			printw("%6.2f%%", 100.00 * ((float) disk_stat_ptr->used / (float) (disk_stat_ptr->used + disk_stat_ptr->avail)));
 			disk_stat_ptr++;
 			line++;
 			if(colors) {
