@@ -16,7 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  *
  * $Id$
  */
@@ -31,22 +32,23 @@
 #include <unistd.h>
 
 typedef enum {
-	LONG_LONG = 0,
-	UNSIGNED_LONG_LONG,
-	BYTES,
-	TIME_T,
-	FLOAT,
-	DOUBLE,
-	STRING,
-	INT,
-	UNSIGNED,
-	BOOL,
-	DUPLEX
+	STAT_TYPE_LONG_LONG = 0,
+	STAT_TYPE_UNSIGNED_LONG_LONG,
+	STAT_TYPE_BYTES,
+	STAT_TYPE_TIME_T,
+	STAT_TYPE_FLOAT,
+	STAT_TYPE_DOUBLE,
+	STAT_TYPE_STRING,
+	STAT_TYPE_INT,
+	STAT_TYPE_UNSIGNED,
+	STAT_TYPE_BOOL,
+	STAT_TYPE_DUPLEX
 } stat_type;
 
 typedef enum {
 	DISPLAY_LINUX = 0,
 	DISPLAY_BSD,
+	DISPLAY_GRAPHITE,
 	DISPLAY_MRTG,
 	DISPLAY_PLAIN
 } display_mode_type;
@@ -79,6 +81,8 @@ int use_cpu_percent = 0;
 int use_diffs = 0;
 long float_scale_factor = 0;
 long long bytes_scale_factor = 0;
+char *graphite_prefix = NULL;
+time_t graphite_ts = 0;
 
 /* Exit with an error message. */
 static void
@@ -130,6 +134,13 @@ add_stat(stat_type type, void *stat, ...) {
 	int len = 0;
 	char *name, *p;
 
+	/* Skip non-numeric values if using Graphite-compatible output */
+	if (display_mode == DISPLAY_GRAPHITE &&
+	    ((type == STAT_TYPE_BOOL) || (type == STAT_TYPE_STRING) ||
+	     (type == STAT_TYPE_DUPLEX))) {
+		return;
+	}
+
 	/* Figure out how long the name will be, including dots and trailing
 	   \0. */
 	va_start(ap, stat);
@@ -201,7 +212,7 @@ populate_const(void) {
 	static int zero = 0;
 
 	/* Constants, for use with MRTG mode. */
-	add_stat(INT, &zero, "const", "0", NULL);
+	add_stat(STAT_TYPE_INT, &zero, "const", "0", NULL);
 }
 
 static void
@@ -218,54 +229,54 @@ populate_cpu(void) {
 		sg_cpu_percents *cpu_p = sg_get_cpu_percents_of(cps, NULL);
 
 		if (cpu_p != NULL) {
-			add_stat(DOUBLE, &cpu_p->user,
+			add_stat(STAT_TYPE_DOUBLE, &cpu_p->user,
 				 "cpu", "user", NULL);
-			add_stat(DOUBLE, &cpu_p->kernel,
+			add_stat(STAT_TYPE_DOUBLE, &cpu_p->kernel,
 				 "cpu", "kernel", NULL);
-			add_stat(DOUBLE, &cpu_p->idle,
+			add_stat(STAT_TYPE_DOUBLE, &cpu_p->idle,
 				 "cpu", "idle", NULL);
-			add_stat(DOUBLE, &cpu_p->iowait,
+			add_stat(STAT_TYPE_DOUBLE, &cpu_p->iowait,
 				 "cpu", "iowait", NULL);
-			add_stat(DOUBLE, &cpu_p->swap,
+			add_stat(STAT_TYPE_DOUBLE, &cpu_p->swap,
 				 "cpu", "swap", NULL);
-			add_stat(DOUBLE, &cpu_p->nice,
+			add_stat(STAT_TYPE_DOUBLE, &cpu_p->nice,
 				 "cpu", "nice", NULL);
-			add_stat(TIME_T, &cpu_p->time_taken,
+			add_stat(STAT_TYPE_TIME_T, &cpu_p->time_taken,
 				 "cpu", "time_taken", NULL);
 		}
 	} else {
 		if (cpu_s != NULL) {
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->user,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->user,
 				 "cpu", "user", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->kernel,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->kernel,
 				 "cpu", "kernel", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->idle,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->idle,
 				 "cpu", "idle", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->iowait,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->iowait,
 				 "cpu", "iowait", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->swap,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->swap,
 				 "cpu", "swap", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->nice,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->nice,
 				 "cpu", "nice", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &cpu_s->total,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->total,
 				 "cpu", "total", NULL);
-			add_stat(TIME_T, &cpu_s->systime,
+			add_stat(STAT_TYPE_TIME_T, &cpu_s->systime,
 				 "cpu", "systime", NULL);
 		}
 	}
 
 	if (cpu_s != NULL) {
-		add_stat(UNSIGNED_LONG_LONG, &cpu_s->context_switches,
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->context_switches,
 			 "cpu", "ctxsw", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &cpu_s->voluntary_context_switches,
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->voluntary_context_switches,
 			 "cpu", "vctxsw", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &cpu_s->involuntary_context_switches,
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->involuntary_context_switches,
 			 "cpu", "nvctxsw", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &cpu_s->syscalls,
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->syscalls,
 			 "cpu", "syscalls", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &cpu_s->interrupts,
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->interrupts,
 			 "cpu", "intrs", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &cpu_s->soft_interrupts,
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &cpu_s->soft_interrupts,
 			 "cpu", "softintrs", NULL);
 	}
 }
@@ -275,10 +286,10 @@ populate_mem(void) {
 	sg_mem_stats *mem = sg_get_mem_stats(NULL);
 
 	if (mem != NULL) {
-		add_stat(BYTES, &mem->total, "mem", "total", NULL);
-		add_stat(BYTES, &mem->free, "mem", "free", NULL);
-		add_stat(BYTES, &mem->used, "mem", "used", NULL);
-		add_stat(BYTES, &mem->cache, "mem", "cache", NULL);
+		add_stat(STAT_TYPE_BYTES, &mem->total, "mem", "total", NULL);
+		add_stat(STAT_TYPE_BYTES, &mem->free, "mem", "free", NULL);
+		add_stat(STAT_TYPE_BYTES, &mem->used, "mem", "used", NULL);
+		add_stat(STAT_TYPE_BYTES, &mem->cache, "mem", "cache", NULL);
 	}
 }
 
@@ -287,9 +298,9 @@ populate_load(void) {
 	sg_load_stats *load = sg_get_load_stats(NULL);
 
 	if (load != NULL) {
-		add_stat(DOUBLE, &load->min1, "load", "min1", NULL);
-		add_stat(DOUBLE, &load->min5, "load", "min5", NULL);
-		add_stat(DOUBLE, &load->min15, "load", "min15", NULL);
+		add_stat(STAT_TYPE_DOUBLE, &load->min1, "load", "min1", NULL);
+		add_stat(STAT_TYPE_DOUBLE, &load->min5, "load", "min5", NULL);
+		add_stat(STAT_TYPE_DOUBLE, &load->min15, "load", "min15", NULL);
 	}
 }
 
@@ -309,13 +320,13 @@ populate_user(void) {
 
 			name_list_length += strlen(name) + 1;
 
-			add_stat(STRING, &users[i].login_name,
+			add_stat(STAT_TYPE_STRING, &users[i].login_name,
 				 "user", tty, "login_name", NULL);
-			add_stat(STRING, &users[i].device,
+			add_stat(STAT_TYPE_STRING, &users[i].device,
 				 "user", tty, "tty", NULL);
-			add_stat(STRING, &users[i].hostname,
+			add_stat(STAT_TYPE_STRING, &users[i].hostname,
 				 "user", tty, "from", NULL);
-			add_stat(TIME_T, &users[i].login_time,
+			add_stat(STAT_TYPE_TIME_T, &users[i].login_time,
 				 "user", tty, "login_time", NULL);
 		}
 
@@ -335,8 +346,8 @@ populate_user(void) {
 		}
 		name_list[pos] = '\0';
 
-		add_stat(INT, &entries, "user", "num", NULL);
-		add_stat(STRING, &name_list, "user", "names", NULL);
+		add_stat(STAT_TYPE_INT, &entries, "user", "num", NULL);
+		add_stat(STAT_TYPE_STRING, &name_list, "user", "names", NULL);
 	}
 }
 
@@ -345,9 +356,9 @@ populate_swap(void) {
 	sg_swap_stats *swap = sg_get_swap_stats(NULL);
 
 	if (swap != NULL) {
-		add_stat(BYTES, &swap->total, "swap", "total", NULL);
-		add_stat(BYTES, &swap->used, "swap", "used", NULL);
-		add_stat(BYTES, &swap->free, "swap", "free", NULL);
+		add_stat(STAT_TYPE_BYTES, &swap->total, "swap", "total", NULL);
+		add_stat(STAT_TYPE_BYTES, &swap->used, "swap", "used", NULL);
+		add_stat(STAT_TYPE_BYTES, &swap->free, "swap", "free", NULL);
 	}
 }
 
@@ -369,21 +380,21 @@ populate_general(void) {
 	sg_host_info *host = sg_get_host_info(NULL);
 
 	if (host != NULL) {
-		add_stat(STRING, &host->os_name,
+		add_stat(STAT_TYPE_STRING, &host->os_name,
 			 "general", "os_name", NULL);
-		add_stat(STRING, &host->os_release,
+		add_stat(STAT_TYPE_STRING, &host->os_release,
 			 "general", "os_release", NULL);
-		add_stat(STRING, &host->os_version,
+		add_stat(STAT_TYPE_STRING, &host->os_version,
 			 "general", "os_version", NULL);
-		add_stat(STRING, &host->platform, "general", "platform", NULL);
-		add_stat(STRING, &host->hostname, "general", "hostname", NULL);
-		add_stat(UNSIGNED, &host->ncpus, "general", "ncpus", NULL);
-		add_stat(UNSIGNED, &host->maxcpus, "general", "ncpus_cfg", NULL);
-		add_stat(UNSIGNED, &host->bitwidth, "general", "bitwidth", NULL);
-		add_stat(STRING, ((size_t)host->host_state) > (lengthof(host_states) - 1)
+		add_stat(STAT_TYPE_STRING, &host->platform, "general", "platform", NULL);
+		add_stat(STAT_TYPE_STRING, &host->hostname, "general", "hostname", NULL);
+		add_stat(STAT_TYPE_UNSIGNED, &host->ncpus, "general", "ncpus", NULL);
+		add_stat(STAT_TYPE_UNSIGNED, &host->maxcpus, "general", "ncpus_cfg", NULL);
+		add_stat(STAT_TYPE_UNSIGNED, &host->bitwidth, "general", "bitwidth", NULL);
+		add_stat(STAT_TYPE_STRING, ((size_t)host->host_state) > (lengthof(host_states) - 1)
 		                 ? &unexpected_host_state
 		                 : &host_states[host->host_state], "general", "hoststate", NULL);
-		add_stat(TIME_T, &host->uptime, "general", "uptime", NULL);
+		add_stat(STAT_TYPE_TIME_T, &host->uptime, "general", "uptime", NULL);
 	}
 }
 
@@ -413,37 +424,39 @@ populate_fs(void) {
 			while ((p = strchr(name, '/')) != NULL)
 				*p = '_';
 
-			add_stat(STRING, &disk[i].device_name,
+			add_stat(STAT_TYPE_STRING, &disk[i].device_name,
 				 "fs", name, "device_name", NULL);
-			add_stat(STRING, &disk[i].fs_type,
+			add_stat(STAT_TYPE_STRING, &disk[i].device_canonical,
+				 "fs", name, "device_canonical", NULL);
+			add_stat(STAT_TYPE_STRING, &disk[i].fs_type,
 				 "fs", name, "fs_type", NULL);
-			add_stat(STRING, &disk[i].mnt_point,
+			add_stat(STAT_TYPE_STRING, &disk[i].mnt_point,
 				 "fs", name, "mnt_point", NULL);
-			add_stat(BYTES, &disk[i].size,
+			add_stat(STAT_TYPE_BYTES, &disk[i].size,
 				 "fs", name, "size", NULL);
-			add_stat(BYTES, &disk[i].used,
+			add_stat(STAT_TYPE_BYTES, &disk[i].used,
 				 "fs", name, "used", NULL);
-			add_stat(BYTES, &disk[i].avail,
+			add_stat(STAT_TYPE_BYTES, &disk[i].avail,
 				 "fs", name, "avail", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].total_inodes,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].total_inodes,
 				 "fs", name, "total_inodes", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].used_inodes,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].used_inodes,
 				 "fs", name, "used_inodes", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].free_inodes,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].free_inodes,
 				 "fs", name, "free_inodes", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].avail_inodes,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].avail_inodes,
 				 "fs", name, "avail_inodes", NULL);
-			add_stat(BYTES, &disk[i].io_size,
+			add_stat(STAT_TYPE_BYTES, &disk[i].io_size,
 				 "fs", name, "io_size", NULL);
-			add_stat(BYTES, &disk[i].block_size,
+			add_stat(STAT_TYPE_BYTES, &disk[i].block_size,
 				 "fs", name, "block_size", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].total_blocks,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].total_blocks,
 				 "fs", name, "total_blocks", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].free_blocks,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].free_blocks,
 				 "fs", name, "free_blocks", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].avail_blocks,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].avail_blocks,
 				 "fs", name, "avail_blocks", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &disk[i].used_blocks,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &disk[i].used_blocks,
 				 "fs", name, "used_blocks", NULL);
 
 			free(buf);
@@ -462,13 +475,13 @@ populate_disk(void) {
 		for (i = 0; i < n; i++) {
 			const char *name = diskio[i].disk_name;
 
-			add_stat(STRING, &diskio[i].disk_name,
+			add_stat(STAT_TYPE_STRING, &diskio[i].disk_name,
 				 "disk", name, "disk_name", NULL);
-			add_stat(BYTES, &diskio[i].read_bytes,
+			add_stat(STAT_TYPE_BYTES, &diskio[i].read_bytes,
 				 "disk", name, "read_bytes", NULL);
-			add_stat(BYTES, &diskio[i].write_bytes,
+			add_stat(STAT_TYPE_BYTES, &diskio[i].write_bytes,
 				 "disk", name, "write_bytes", NULL);
-			add_stat(TIME_T, &diskio[i].systime,
+			add_stat(STAT_TYPE_TIME_T, &diskio[i].systime,
 				 "disk", name, "systime", NULL);
 		}
 	}
@@ -480,11 +493,11 @@ populate_proc(void) {
 	sg_process_count *proc = sg_get_process_count();
 
 	if (proc != NULL) {
-		add_stat(UNSIGNED_LONG_LONG, &proc->total, "proc", "total", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &proc->running, "proc", "running", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &proc->sleeping, "proc", "sleeping", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &proc->stopped, "proc", "stopped", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &proc->zombie, "proc", "zombie", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &proc->total, "proc", "total", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &proc->running, "proc", "running", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &proc->sleeping, "proc", "sleeping", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &proc->stopped, "proc", "stopped", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &proc->zombie, "proc", "zombie", NULL);
 	}
 	else {
 		char *errbuf;
@@ -509,23 +522,23 @@ populate_net(void) {
 		for (i = 0; i < num_io; i++) {
 			const char *name = io[i].interface_name;
 
-			add_stat(STRING, &io[i].interface_name,
+			add_stat(STAT_TYPE_STRING, &io[i].interface_name,
 				 "net", name, "interface_name", NULL);
-			add_stat(BYTES, &io[i].tx,
+			add_stat(STAT_TYPE_BYTES, &io[i].tx,
 				 "net", name, "tx", NULL);
-			add_stat(BYTES, &io[i].rx,
+			add_stat(STAT_TYPE_BYTES, &io[i].rx,
 				 "net", name, "rx", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &io[i].ipackets,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &io[i].ipackets,
 				 "net", name, "ipackets", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &io[i].opackets,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &io[i].opackets,
 				 "net", name, "opackets", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &io[i].ierrors,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &io[i].ierrors,
 				 "net", name, "ierrors", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &io[i].oerrors,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &io[i].oerrors,
 				 "net", name, "oerrors", NULL);
-			add_stat(UNSIGNED_LONG_LONG, &io[i].collisions,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &io[i].collisions,
 				 "net", name, "collisions", NULL);
-			add_stat(TIME_T, &io[i].systime,
+			add_stat(STAT_TYPE_TIME_T, &io[i].systime,
 				 "net", name, "systime", NULL);
 		}
 	}
@@ -549,15 +562,15 @@ populate_net(void) {
 				}
 			}
 			if (!had_io) {
-				add_stat(STRING, &iface[i].interface_name,
-				 	"net", name, "interface_name", NULL);
+				add_stat(STAT_TYPE_STRING, &iface[i].interface_name,
+					"net", name, "interface_name", NULL);
 			}
 
-			add_stat(UNSIGNED_LONG_LONG, &iface[i].speed,
+			add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &iface[i].speed,
 				 "net", name, "speed", NULL);
-			add_stat(BOOL, &iface[i].up,
+			add_stat(STAT_TYPE_BOOL, &iface[i].up,
 				 "net", name, "up", NULL);
-			add_stat(DUPLEX, &iface[i].duplex,
+			add_stat(STAT_TYPE_DUPLEX, &iface[i].duplex,
 				 "net", name, "duplex", NULL);
 		}
 	}
@@ -569,9 +582,9 @@ populate_page(void) {
 
 	page = use_diffs ? sg_get_page_stats_diff(NULL) : sg_get_page_stats(NULL);
 	if (page != NULL) {
-		add_stat(UNSIGNED_LONG_LONG, &page->pages_pagein, "page", "in", NULL);
-		add_stat(UNSIGNED_LONG_LONG, &page->pages_pageout, "page", "out", NULL);
-		add_stat(TIME_T, &page->systime, "page", "systime", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &page->pages_pagein, "page", "in", NULL);
+		add_stat(STAT_TYPE_UNSIGNED_LONG_LONG, &page->pages_pageout, "page", "out", NULL);
+		add_stat(STAT_TYPE_TIME_T, &page->systime, "page", "systime", NULL);
 	}
 }
 
@@ -634,6 +647,8 @@ get_stats(void) {
 
 	if (stat_items != NULL)
 		qsort(stat_items, num_stats, sizeof *stat_items, stats_compare);
+	if (display_mode == DISPLAY_GRAPHITE)
+		graphite_ts = time(NULL);
 }
 
 /* Print the value of a stat_item. */
@@ -649,21 +664,21 @@ print_stat_value(const stat_item *s) {
 	} v;
 
 	switch (s->type) {
-	case LONG_LONG:
+	case STAT_TYPE_LONG_LONG:
 #ifdef WIN32 /* Windows printf does not understand %lld, so use %I64d instead */
 		printf("%I64d", *(long long *)pv);
 #else
 		printf("%lld", *(long long *)pv);
 #endif
 		break;
-	case UNSIGNED_LONG_LONG:
+	case STAT_TYPE_UNSIGNED_LONG_LONG:
 #ifdef WIN32 /* Windows printf does not understand %llu, so use %I64u instead */
 		printf("%I64u", *(unsigned long long *)pv);
 #else
 		printf("%llu", *(unsigned long long *)pv);
 #endif
 		break;
-	case BYTES:
+	case STAT_TYPE_BYTES:
 		v.ull = *(unsigned long long *)pv;
 		if (bytes_scale_factor != 0) {
 			v.ull >>= (bytes_scale_factor-1);
@@ -676,14 +691,14 @@ print_stat_value(const stat_item *s) {
 		printf("%llu", v.ull);
 #endif
 		break;
-	case TIME_T:
+	case STAT_TYPE_TIME_T:
 		/* FIXME option for formatted time? */
 		v.l = *(time_t *)pv;
 		printf("%ld", v.l);
 		break;
-	case FLOAT:
-	case DOUBLE:
-		if (s->type == FLOAT) {
+	case STAT_TYPE_FLOAT:
+	case STAT_TYPE_DOUBLE:
+		if (s->type == STAT_TYPE_FLOAT) {
 			v.f = *(float *)pv;
 		}
 		else {
@@ -696,20 +711,20 @@ print_stat_value(const stat_item *s) {
 			printf("%f", v.f);
 		}
 		break;
-	case STRING:
+	case STAT_TYPE_STRING:
 		/* FIXME escaping? */
 		printf("%s", *(char **)pv);
 		break;
-	case INT:
+	case STAT_TYPE_INT:
 		printf("%d", *(int *)pv);
 		break;
-	case UNSIGNED:
+	case STAT_TYPE_UNSIGNED:
 		printf("%u", *(unsigned *)pv);
 		break;
-	case BOOL:
+	case STAT_TYPE_BOOL:
 		printf("%s", *(int *)pv ? "true" : "false");
 		break;
-	case DUPLEX:
+	case STAT_TYPE_DUPLEX:
 		switch (*(sg_iface_duplex *) pv) {
 		case SG_IFACE_DUPLEX_FULL:
 			printf("full");
@@ -735,11 +750,20 @@ print_stat(const stat_item *s) {
 	case DISPLAY_BSD:
 		printf("%s: ", s->name);
 		break;
+	case DISPLAY_GRAPHITE:
+		if (graphite_prefix == NULL) {
+			printf("%s ", s->name);
+		} else {
+			printf("%s.%s ", graphite_prefix, s->name);
+		}
+		break;
 	case DISPLAY_MRTG:
 	case DISPLAY_PLAIN:
 		break;
 	}
 	print_stat_value(s);
+	if (display_mode == DISPLAY_GRAPHITE)
+		printf(" "FMT_TIME_T, graphite_ts);
 	printf("\n");
 }
 
@@ -797,6 +821,10 @@ print_stats(int argc, char **argv) {
 			}
 		}
 	}
+
+#if !defined(HAVE_SETVBUF) && !defined(HAVE_SETLINEBUF)
+	fflush(stdout);
+#endif
 }
 
 static void
@@ -810,6 +838,8 @@ usage(void) {
 	printf("  -l         Linux sysctl-style output (default)\n"
 	       "  -b         BSD sysctl-style output\n"
 	       "  -m         MRTG-compatible output\n"
+	       "  -r         Graphite-compatible output\n"
+	       "  -R PREFIX  Graphite-compatible output with STATs prefixed by 'PREFIX.'\n"
 	       "  -u         Plain output (only show values)\n"
 	       "  -n         Display cumulative stats once (default)\n"
 	       "  -s         Display stat differences repeatedly\n"
@@ -962,9 +992,20 @@ int
 main(int argc, char **argv) {
 	char *fslist = NULL;
 
+#if defined(HAVE_SETVBUF)
+# ifndef BUFSIZ
+#  define BUFSIZ 256
+# endif
+	int rc = setvbuf(stdout, NULL, _IOLBF, BUFSIZ);
+	if(0 != rc)
+		die(strerror(errno));
+#elif defined(HAVE_SETLINEBUF)
+	setlinebuf(stdout);
+#endif
+
 	opterr = 0;
 	while (1) {
-		int c = getopt(argc, argv, "lbmunsot:pf:F:KMG");
+		int c = getopt(argc, argv, "lbmrR:unsot:pf:F:KMG");
 		if (c == -1)
 			break;
 		switch (c) {
@@ -977,6 +1018,13 @@ main(int argc, char **argv) {
 		case 'm':
 			display_mode = DISPLAY_MRTG;
 			break;
+		case 'r':
+			display_mode = DISPLAY_GRAPHITE;
+			break;
+		case 'R':
+			display_mode = DISPLAY_GRAPHITE;
+			graphite_prefix = optarg;
+			break;
 		case 'u':
 			display_mode = DISPLAY_PLAIN;
 			break;
